function [y,T,lam,res,iter,A,p,jac]=honloopext(p,y1,T1,lam1,ds,varargin)
% honloopext: Newton loop for Hopf in arclength
if p.fsol.fsol==1; % use fsolve 
    [y,T,lam,res,iter,A,p,jac]=fsolext(p,y1,T1,lam1,ds); return; 
end
iter=0; y=y1; T=T1; lam=lam1; p=setlam(p,lam); % set lam to predictor
nu=p.nu; tl=p.hopf.tl; alpha=1; almin=alpha/4; % minimal damping 
[f,jac,f_T,f_lam]=tomassempbc(p,y,T,lam); 
[pc,pc_y]=hopc(p,y,T,lam); arcl=hoarcl(p,y,T,lam,ds); F=[f; pc; arcl]; 
res0=norm(F,p.sw.norm); res=res0; 
if p.sw.verb>1; 
    fprintf('initial (||f||, pc, arcl)=(%g,%g,%g)\n',norm(f,p.sw.norm),pc,arcl); end
A=gethoA(p,jac,f_T,f_lam,pc_y,p.hopf.tau); 
tol=p.nc.tol; 
if(res<tol); return; end; % initial res. small, do nothing! 
% now start the actual loop
stepok=1; % stepok=1 indicates that step of size alpha decreased residual  
while(abs(res0)>tol && iter<p.nc.imax && stepok) 
  [upd,p]=p.fuha.blss(A,F,p); 
  stepok=0; yv=reshape(y(1:nu,:),nu*tl,1); % make y-vector
  while(stepok==0 && alpha>=almin)   
  y1=yv-alpha*upd(1:nu*tl); tupd=upd(nu*tl+1); T1=T-alpha*tupd; % res 
  lam1=lam-alpha*upd(nu*tl+2); 
  ytom=reshape(y1(1:nu*tl),nu,tl); % convert to tom-input 
  f=tomassempbc(p,ytom,T1,lam1);  % setlam(p,lam1) done in tomassempbc! 
  [pc,pc_y]=hopc(p,ytom,T1,lam1); arcl=hoarcl(p,ytom,T1,lam1,ds); 
  F=[f; pc; arcl];
  if p.sw.verb>1; 
      fprintf('al=%g, (||f||, pc, arcl)=(%g,%g,%g)\n',alpha,norm(f,p.sw.norm),pc,arcl);end
  res=norm(F,p.sw.norm); % check new tom-resi 
  if(res<res0); % good step       
    if(res<res0/2 && alpha<1) alpha=alpha*2; end % very good step  
    stepok=1; y=ytom; T=T1; lam=lam1; res0=res;
    [f,jac,f_T,f_lam]=tomassempbc(p,y,T,lam); 
    [pc,pc_y]=hopc(p,y,T,lam); 
    A=gethoA(p,jac,f_T,f_lam,pc_y,p.hopf.tau); 
  else alpha=alpha/2; % bad step, try smaller alpha
  end % res<res0
  end % while stepok==0
  iter=iter+1;
end % while res<p.nc.tol && stepok 
% ----------------------------------------------------------postprocessing
if(p.sw.verb>1); if(alpha<1) fprintf('\nhonloopext: damp alpha=%g, res=%g, ds=%g\n',...
            alpha,res,ds); end; end; % inform user if damping was used 
res=res0; y=ytom;

